@@ -30,8 +30,6 @@
 
 #include "window-list.h"
 
-#include "egg-screen-help.h"
-
 typedef struct {
 	GtkWidget *applet;
 	GtkWidget *tasklist;
@@ -104,7 +102,7 @@ response_cb (GtkWidget    *widget,
 	if (id == GTK_RESPONSE_HELP) {
 		GError *error = NULL;
 
-		egg_help_display_desktop_on_screen (
+		gnome_help_display_desktop_on_screen (
 			NULL, "window-list", "window-list", "windowlist-prefs",
 			gtk_widget_get_screen (tasklist->applet),
 			&error);
@@ -761,7 +759,7 @@ display_help_dialog (BonoboUIComponent *uic,
 {
 	GError *error = NULL;
 
-	egg_help_display_desktop_on_screen (
+	gnome_help_display_desktop_on_screen (
 			NULL, "window-list", "window-list", NULL,
 			gtk_widget_get_screen (tasklist->applet),
 			&error);
@@ -800,7 +798,8 @@ display_about_dialog (BonoboUIComponent *uic,
 		NULL
 	};
 	const char *documenters [] = {
-	  NULL
+		"Sun GNOME Documentation Team <gdocteam@sun.com>",
+		NULL
 	};
 	const char *translator_credits = _("translator_credits");
 
