@@ -31,14 +31,16 @@
 
 #include "workspace-switcher.h"
 
-#include "egg-screen-help.h"
-
 #include "wncklet.h"
 
 /* even 16 is pretty darn dubious. */
 #define MAX_REASONABLE_ROWS 16
 #define DEFAULT_ROWS 1
 
+#define NEVER_SENSITIVE "never_sensitive"
+#define NUM_WORKSPACES "/apps/metacity/general/num_workspaces"
+#define WORKSPACE_NAME "/apps/metacity/workspace_names/name_1"
+
 typedef struct {
 	GtkWidget *applet;
 
@@ -153,7 +155,7 @@ response_cb (GtkWidget *widget,
 	if (id == GTK_RESPONSE_HELP) {
 		GError *error = NULL;
 
-		egg_help_display_desktop_on_screen (
+		gnome_help_display_desktop_on_screen (
 			NULL, "workspace-switcher", "workspace-switcher", "workspacelist-prefs",
 			gtk_widget_get_screen (pager->applet),
 			&error);
@@ -191,6 +193,8 @@ destroy_pager(GtkWidget * widget, PagerData *pager)
 	gconf_client_notify_remove (client, pager->listeners[1]);
 	gconf_client_notify_remove (client, pager->listeners[2]);
 
+	g_object_unref (G_OBJECT (client));
+
 	pager->listeners[0] = 0;
 	pager->listeners[1] = 0;
 	pager->listeners[2] = 0;
@@ -286,7 +290,8 @@ all_workspaces_changed (GConfClient *client,
 				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (pager->current_only_radio), TRUE);
 			}
 		}
-		gtk_widget_set_sensitive (pager->num_rows_spin, value);
+		if ( ! g_object_get_data (G_OBJECT (pager->num_rows_spin), NEVER_SENSITIVE))
+			gtk_widget_set_sensitive (pager->num_rows_spin, value);
 	}
 }
 
@@ -326,6 +331,8 @@ setup_gconf (PagerData *pager)
 		
 	g_free (key);
 
+	g_object_unref (G_OBJECT (client));
+
 }
 
 gboolean
@@ -429,6 +436,17 @@ workspace_switcher_applet_fill (PanelApplet *applet)
 					   pager_menu_verbs,
 					   pager);
 
+	if (panel_applet_get_locked_down (PANEL_APPLET (pager->applet))) {
+		BonoboUIComponent *popup_component;
+
+		popup_component = panel_applet_get_popup_component (PANEL_APPLET (pager->applet));
+
+		bonobo_ui_component_set_prop (popup_component,
+					      "/commands/PagerPreferences",
+					      "hidden", "1",
+					      NULL);
+	}
+
 	return TRUE;
 }
 
@@ -440,7 +458,7 @@ display_help_dialog (BonoboUIComponent *uic,
 {
 	GError *error = NULL;
 
-	egg_help_display_desktop_on_screen (
+	gnome_help_display_desktop_on_screen (
 		NULL, "workspace-switcher", "workspace-switcher", NULL,
 		gtk_widget_get_screen (pager->applet),
 		&error);
@@ -481,7 +499,9 @@ display_about_dialog (BonoboUIComponent *uic,
 		NULL
 	};
 	const char *documenters [] = {
-	  NULL
+                "John Fleck <jfleck@inkstain.net>",
+                "Sun GNOME Documentation Team <gdocteam@sun.com>",
+                NULL
 	};
 	const char *translator_credits = _("translator_credits");
 
@@ -612,6 +632,18 @@ num_workspaces_value_changed (GtkSpinButton *button,
                                             gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (pager->num_workspaces_spin)));
 }
 
+static gboolean
+workspaces_tree_focused_out (GtkTreeView   *treeview,
+			     GdkEventFocus *event,
+			     PagerData     *pager)
+{
+	GtkTreeSelection *selection;
+
+	selection = gtk_tree_view_get_selection (treeview);
+	gtk_tree_selection_unselect_all (selection);
+	return TRUE;
+}
+
 static void 
 workspace_name_edited (GtkCellRendererText *cell_renderer_text,
 		       const gchar         *path,
@@ -671,6 +703,55 @@ close_dialog (GtkWidget *button,
 	gtk_widget_hide (pager->properties_dialog);
 }
 
+static void
+setup_sensitivity (PagerData *pager,
+		   GladeXML *xml,
+		   const char *wid1,
+		   const char *wid2,
+		   const char *wid3,
+		   const char *key)
+{
+	PanelApplet *applet = PANEL_APPLET (pager->applet);
+	GConfClient *client = gconf_client_get_default ();
+	char *fullkey;
+	GtkWidget *w;
+
+	if (key[0] == '/')
+		fullkey = g_strdup (key);
+	else
+		fullkey = panel_applet_gconf_get_full_key (applet, key);
+
+	if (gconf_client_key_is_writable (client, fullkey, NULL)) {
+		g_object_unref (G_OBJECT (client));
+		g_free (fullkey);
+		return;
+	}
+	g_object_unref (G_OBJECT (client));
+	g_free (fullkey);
+
+	w = glade_xml_get_widget (xml, wid1);
+	g_assert (w != NULL);
+	g_object_set_data (G_OBJECT (w), NEVER_SENSITIVE,
+			   GINT_TO_POINTER (1));
+	gtk_widget_set_sensitive (w, FALSE);
+
+	if (wid2 != NULL) {
+		w = glade_xml_get_widget (xml, wid2);
+		g_assert (w != NULL);
+		g_object_set_data (G_OBJECT (w), NEVER_SENSITIVE,
+				   GINT_TO_POINTER (1));
+		gtk_widget_set_sensitive (w, FALSE);
+	}
+	if (wid3 != NULL) {
+		w = glade_xml_get_widget (xml, wid3);
+		g_assert (w != NULL);
+		g_object_set_data (G_OBJECT (w), NEVER_SENSITIVE,
+				   GINT_TO_POINTER (1));
+		gtk_widget_set_sensitive (w, FALSE);
+	}
+
+}
+
 static void
 setup_dialog (GladeXML  *xml,
 	      PagerData *pager)
@@ -681,12 +762,41 @@ setup_dialog (GladeXML  *xml,
 	int nr_ws, i;
 	
 	pager->display_workspaces_toggle = WID ("workspace_name_toggle");
+	setup_sensitivity (pager, xml,
+			   "workspace_name_toggle",
+			   NULL,
+			   NULL,
+			   "display_workspace_names" /* key */);
+
 	pager->all_workspaces_radio = WID ("all_workspaces_radio");
 	pager->current_only_radio = WID ("current_only_radio");
+	setup_sensitivity (pager, xml,
+			   "all_workspaces_radio",
+			   "current_only_radio",
+			   "label_row_col",
+			   "display_all_workspaces" /* key */);
+
 	pager->num_rows_spin = WID ("num_rows_spin");
 	pager->label_row_col = WID("label_row_col");
+	setup_sensitivity (pager, xml,
+			   "num_rows_spin",
+			   NULL,
+			   NULL,
+			   "num_rows" /* key */);
+
 	pager->num_workspaces_spin = WID ("num_workspaces_spin");
+	setup_sensitivity (pager, xml,
+			   "num_workspaces_spin",
+			   NULL,
+			   NULL,
+			   NUM_WORKSPACES /* key */);
+
 	pager->workspaces_tree = WID ("workspaces_tree_view");
+	setup_sensitivity (pager, xml,
+			   "workspaces_tree_view",
+			   NULL,
+			   NULL,
+			   WORKSPACE_NAME /* key */);
 
 	/* Display workspace names: */
 	
@@ -707,7 +817,8 @@ setup_dialog (GladeXML  *xml,
 
 	if (pager->display_all) {
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (pager->all_workspaces_radio), TRUE);
-		gtk_widget_set_sensitive (pager->num_rows_spin, TRUE);
+		if ( ! g_object_get_data (G_OBJECT (pager->num_rows_spin), NEVER_SENSITIVE))
+			gtk_widget_set_sensitive (pager->num_rows_spin, TRUE);
 	} else {
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (pager->current_only_radio), TRUE);
 		gtk_widget_set_sensitive (pager->num_rows_spin, FALSE);
@@ -745,6 +856,9 @@ setup_dialog (GladeXML  *xml,
 				     pager,
 				     pager->applet);
 
+	g_signal_connect (G_OBJECT (pager->workspaces_tree), "focus_out_event",
+			  (GCallback) workspaces_tree_focused_out, pager);
+
 	pager->workspaces_store = gtk_list_store_new (1, G_TYPE_STRING, NULL);
 	update_workspaces_model (pager);
 	gtk_tree_view_set_model (GTK_TREE_VIEW (pager->workspaces_tree), GTK_TREE_MODEL (pager->workspaces_store));
