@@ -45,7 +45,11 @@ free_prepared_resources (PanelBackground *background)
 	case PANEL_BACK_NONE:
 		break;
 	case PANEL_BACK_COLOR:
-		if (!background->has_alpha) {
+		if (background->has_alpha) {
+			if (background->pixmap)
+				g_object_unref (background->pixmap);
+			background->pixmap = NULL;
+		} else {
 			if (background->colormap && background->color.gdk.pixel)
 				gdk_colormap_free_colors (
 					background->colormap,
@@ -121,8 +125,14 @@ panel_background_prepare (PanelBackground *background)
 		break;
 	}
 
+	/* Panel applets may use the panel's background pixmap to
+	 * decide how to draw themselves.  Therefore, we need to
+	 * make sure that all drawing has been completed before
+	 * the applet looks at the pixmap. */
+	gdk_display_sync (gdk_drawable_get_display (background->window));
+
 	gdk_window_get_user_data (GDK_WINDOW (background->window),
-				  (gpointer *) &widget);
+				  (gpointer) &widget);
 
 	if (GTK_IS_WIDGET (widget))
 	  gtk_widget_queue_draw (widget);
@@ -202,6 +212,9 @@ composite_image_onto_desktop (PanelBackground *background)
 	if (!background->desktop)
 		background->desktop = get_desktop_pixbuf (background);
 
+	if (!background->desktop)
+		return NULL;
+
 	retval = gdk_pixbuf_copy (background->desktop);
 
 	width  = gdk_pixbuf_get_width  (background->desktop);
@@ -231,6 +244,9 @@ composite_color_onto_desktop (PanelBackground *background)
 	if (!background->desktop)
 		background->desktop = get_desktop_pixbuf (background);
 
+	if (!background->desktop)
+		return NULL;
+
 	color = ((background->color.gdk.red & 0xff00) << 8) +
 		 (background->color.gdk.green & 0xff00) +
 		 (background->color.gdk.blue >> 8);
@@ -303,6 +319,8 @@ panel_background_composite (PanelBackground *background)
 
 	panel_background_prepare (background);
 
+	background->notify_changed (background, background->user_data);
+
 	return TRUE;
 }
 
@@ -370,45 +388,73 @@ get_scaled_and_rotated_pixbuf (PanelBackground *background)
 			width  = panel_width;
 			height = panel_height;
 		}
+	} else if (background->orientation == GTK_ORIENTATION_VERTICAL &&
+		   background->rotate_image) {
+		int tmp = width;
+		width = height;
+		height = tmp;
 	}
 
-	scaled = gdk_pixbuf_scale_simple (
-			background->loaded_image,
-			width, height,
-			GDK_INTERP_BILINEAR);
+	if (width == orig_width &&
+	    height == orig_height) {
+		scaled = background->loaded_image;
+		g_object_ref (scaled);
+	} else {
+		scaled = gdk_pixbuf_scale_simple (
+				background->loaded_image,
+				width, height,
+				GDK_INTERP_BILINEAR);
+	}
 
 	if (background->rotate_image &&
 	    background->orientation == GTK_ORIENTATION_VERTICAL) {
-		gulong *dest;
-		gulong *src;
-		int     x, y;
-
-		/* FIXME: horrible lazy hack.
-		 *        The rotation loop below was optimised
-		 *        for 32 bit pixel format pixbufs. Need
-		 *        a version that can handle pixbufs without
-		 *        and alpha channel.
-		 */
 		if (!background->has_alpha) {
-			GdkPixbuf *tmp;
+			guchar *dest;
+			guchar *src;
+			int     x, y;
+			int     destrowstride;
+			int     srcrowstride;
+
+			retval = gdk_pixbuf_new (
+				GDK_COLORSPACE_RGB, FALSE, 8, height, width);
+
+			dest          = gdk_pixbuf_get_pixels (retval);
+			destrowstride = gdk_pixbuf_get_rowstride (retval);
+			src           = gdk_pixbuf_get_pixels (scaled);
+			srcrowstride  = gdk_pixbuf_get_rowstride (scaled);
+
+			for (y = 0; y < height; y++)
+				for (x = 0; x < width; x++) {
+					guchar *dstptr = & ( dest [3*y + destrowstride * (width - x - 1)] );
+					guchar *srcptr = & ( src [y * srcrowstride + 3*x] );
+					dstptr[0] = srcptr[0];
+					dstptr[1] = srcptr[1];
+					dstptr[2] = srcptr[2];
+				}
 
-			tmp = gdk_pixbuf_add_alpha (scaled, FALSE, 0, 0, 0);
 			g_object_unref (scaled);
-			scaled = tmp;
-		}
+		} else {
+			guint32 *dest;
+			guint32 *src;
+			int     x, y;
+			int     destrowstride;
+			int     srcrowstride;
 
-		retval = gdk_pixbuf_new (
+			retval = gdk_pixbuf_new (
 				GDK_COLORSPACE_RGB, TRUE, 8, height, width);
 
-		dest = (gulong *) gdk_pixbuf_get_pixels (retval);
-		src  = (gulong *) gdk_pixbuf_get_pixels (scaled);
+			dest          = (guint32 *) gdk_pixbuf_get_pixels (retval);
+			destrowstride =             gdk_pixbuf_get_rowstride (retval) / 4;
+			src           = (guint32 *) gdk_pixbuf_get_pixels (scaled);
+			srcrowstride  =             gdk_pixbuf_get_rowstride (scaled) / 4;
 
-		for (y = 0; y < height; y++)
-			for (x = 0; x < width; x++)
-				dest [y + height * (width - x - 1)] =
-					src [y * width + x];
+			for (y = 0; y < height; y++)
+				for (x = 0; x < width; x++)
+					dest [y + destrowstride * (width - x - 1)] =
+						src [y * srcrowstride + x];
 
-		g_object_unref (scaled);
+			g_object_unref (scaled);
+		}
 	} else
 		retval = scaled;
 
@@ -477,7 +523,7 @@ load_background_file (PanelBackground *background)
 		gdk_pixbuf_new_from_file (background->image, &error);
 	if (!background->loaded_image) {
 		g_assert (error != NULL);
-		g_warning (G_STRLOC ": unable to open '%s' : %s",
+		g_warning (G_STRLOC ": unable to open '%s': %s",
 			   background->image, error->message);
 		g_error_free (error);
 	}
@@ -503,6 +549,8 @@ void
 panel_background_set_pango_color (PanelBackground *background,
 				  PangoColor      *pango_color)
 {
+	g_return_if_fail (pango_color != NULL);
+
 	if (background->color.gdk.red   == pango_color->red &&
 	    background->color.gdk.green == pango_color->green &&
 	    background->color.gdk.blue  == pango_color->blue)
@@ -539,6 +587,8 @@ panel_background_set_color (PanelBackground *background,
 {
 	PangoColor pango_color;
 
+	g_return_if_fail (color != NULL);
+
 	pango_color.red   = color->gdk.red;
 	pango_color.green = color->gdk.green;
 	pango_color.blue  = color->gdk.blue;
@@ -573,6 +623,8 @@ panel_background_set_image (PanelBackground *background,
 	}
 
 	panel_background_update_has_alpha (background);
+
+	panel_background_transform (background);
 }
 
 void
@@ -645,6 +697,8 @@ panel_background_set_default_style (PanelBackground *background,
 				    GdkColor        *color,
 				    GdkPixmap       *pixmap)
 {
+	g_return_if_fail (color != NULL);
+
 	background->default_color = *color;
 
 	if (pixmap)
@@ -663,6 +717,8 @@ void
 panel_background_realized (PanelBackground *background,
 			   GdkWindow       *window)
 {
+	g_return_if_fail (window != NULL);
+
 	if (background->window && background->colormap && background->gc)
 		return;
 
@@ -705,6 +761,9 @@ panel_background_change_region (PanelBackground *background,
 				int              width,
 				int              height)
 {
+	gboolean need_to_retransform = FALSE;
+	gboolean need_to_reprepare = FALSE;
+
 	if (background->region.x == x &&
 	    background->region.y == y &&
 	    background->region.width == width &&
@@ -712,6 +771,31 @@ panel_background_change_region (PanelBackground *background,
 	    background->orientation == orientation)
 		return;
 
+	/* we only need to retransform anything
+	   on size/orientation changes if the
+	   background is an image and some
+	   conditions are met */
+	if (background->type == PANEL_BACK_IMAGE) {
+		if (background->orientation != orientation &&
+		    background->rotate_image) {
+			/* if orientation changes and we are rotating */
+			need_to_retransform = TRUE;
+		} else if ((background->region.width != width ||
+			    background->region.height != height) &&
+			   (background->fit_image ||
+			    background->stretch_image)) {
+			/* or if the size changes and we are 
+			   stretching or fitting the image */
+			need_to_retransform = TRUE;
+		}
+	}
+
+	/* if size changed, we at least need
+	   to "prepare" the background again */
+	if (background->region.width != width ||
+	    background->region.height != height)
+		need_to_reprepare = TRUE;
+
 	background->region.x      = x;
 	background->region.y      = y;
 	background->region.width  = width;
@@ -723,13 +807,28 @@ panel_background_change_region (PanelBackground *background,
 		g_object_unref (background->desktop);
 	background->desktop = NULL;
 
-	panel_background_transform (background);
+	if (need_to_retransform || ! background->transformed)
+		/* only retransform the background if we have in
+		   fact changed size/orientation */
+		panel_background_transform (background);
+	else if (background->has_alpha || ! background->composited)
+		/* only do compositing if we have some alpha
+		   value to worry about */
+		panel_background_composite (background);
+	else if (need_to_reprepare)
+		/* at least we must prepare the background
+		   if the size changed */
+		panel_background_prepare (background);
 }
 
 void
-panel_background_init (PanelBackground *background)
+panel_background_init (PanelBackground              *background,
+		       PanelBackgroundChangedNotify  notify_changed,
+		       gpointer                      user_data)
 {
 	background->type = PANEL_BACK_NONE;
+	background->notify_changed = notify_changed;
+	background->user_data = user_data;
 
 	background->color.gdk.red   = 0;
 	background->color.gdk.blue  = 0;
@@ -814,15 +913,14 @@ panel_background_make_string (PanelBackground *background,
 	char *retval = NULL;
 
 	if (background->type == PANEL_BACK_IMAGE ||
-	    (background->type == PANEL_BACK_COLOR && background->has_alpha) ||
-	    (background->type == PANEL_BACK_NONE && background->default_pixmap)) {
+	    (background->type == PANEL_BACK_COLOR && background->has_alpha)) {
 		GdkNativeWindow pixmap_xid;
 
-		if (!background->window)
+		if (!background->pixmap)
 			return NULL;
 
 		pixmap_xid = gdk_x11_drawable_get_xid (
-				GDK_DRAWABLE (background->window));
+				GDK_DRAWABLE (background->pixmap));
 
 		retval = g_strdup_printf ("pixmap:%d,%d,%d", pixmap_xid, x, y);
 
