@@ -54,6 +54,7 @@
 #include "menu.h"
 #include "menu-fentry.h"
 #include "panel-lockdown.h"
+#include "panel-xutils.h"
 
 #define ICON_SIZE 48.0
 
@@ -75,7 +76,7 @@ typedef struct {
 	
 	GtkListStore     *program_list_store;
 
-	GList            *executables;
+	GList		 *completion_items;
 	GCompletion      *completion;
 	
 	GSList           *add_icon_paths;
@@ -106,7 +107,7 @@ static void
 panel_run_dialog_destroy (PanelRunDialog *dialog)
 {
 	GList *l;
-
+	
 	g_object_unref (dialog->program_list_box);
 	
 	g_slist_foreach (dialog->add_icon_paths, (GFunc) gtk_tree_path_free, NULL);
@@ -133,11 +134,11 @@ panel_run_dialog_destroy (PanelRunDialog *dialog)
 					    dialog->content_notify_id);
 	dialog->content_notify_id = 0;
 
-	for (l = dialog->executables; l; l = l->next)
+	for (l = dialog->completion_items; l; l = l->next)
 		g_free (l->data);
-	g_list_free (dialog->executables);
-	dialog->executables = NULL;
-
+	g_list_free (dialog->completion_items);
+	dialog->completion_items = NULL;
+	
 	if (dialog->completion)
 		g_completion_free (dialog->completion);
 	dialog->completion = NULL;
@@ -178,7 +179,7 @@ panel_run_dialog_set_icon (PanelRunDialog *dialog,
 						     0 /* flags */);
 						     
 	if (icon)
-		pixbuf = gdk_pixbuf_new_from_file (icon, NULL);
+		pixbuf = gdk_pixbuf_new_from_file_at_size (icon, ICON_SIZE, ICON_SIZE, NULL);
 
                         
 	if (pixbuf) {
@@ -233,9 +234,12 @@ command_is_executable (const char *command)
 		return FALSE;
 	
 	/* If we pass an absolute path to g_find_program it just returns
-	 * that absolute path without checking if it is executable.
+	 * that absolute path without checking if it is executable. Also
+	 * make sure its a regular file so we don't try to launch
+	 * directories or device nodes.
 	 */
-	if (!g_file_test (path, G_FILE_TEST_IS_EXECUTABLE)) {
+	if (!g_file_test (path, G_FILE_TEST_IS_EXECUTABLE) ||
+	    !g_file_test (path, G_FILE_TEST_IS_REGULAR)) {
 		g_free (path);
 		return FALSE;
 	}
@@ -455,7 +459,8 @@ panel_run_dialog_append_file_utf8 (PanelRunDialog *dialog,
 	const char *text;
 	char       *quoted, *temp;
 	
-	if (!file)
+	/* Don't allow filenames beginning with '-' */
+	if (!file || file[0] == '-')
 		return;
 	
 	quoted = quote_string (file);
@@ -1068,9 +1073,52 @@ panel_run_dialog_setup_file_button (PanelRunDialog *dialog,
 			  dialog);
 }
 
-static void
-panel_run_dialog_fill_executables_from (PanelRunDialog *dialog,
-					const char     *dirname)
+static GList *
+fill_files_from (GList      *list,
+		 const char *dirname,
+		 const char *prefix,
+		 GList      *existing_items)
+{
+	struct dirent *dent;
+	DIR           *dir;
+	
+	dir = opendir (dirname);
+	
+	if (!dir)
+		return list;
+	
+	while ((dent = readdir (dir))) {
+		char          *file;
+		char          *item;
+		const char    *suffix;
+		
+		file = g_build_filename (dirname, dent->d_name, NULL);
+		
+		suffix = NULL;
+		if (g_file_test (file, G_FILE_TEST_IS_DIR))
+			suffix = "/";
+		
+		g_free (file);
+		
+		item = g_strconcat (prefix, dent->d_name, suffix, NULL);
+		
+		if (g_list_find_custom (existing_items, item, (GCompareFunc) strcmp)) {
+			g_free (item);
+			closedir (dir);
+			return list;
+		}
+		
+		list = g_list_prepend (list, item);
+	}
+
+	closedir (dir);
+	
+	return list;
+}	
+
+static GList *
+fill_executables_from (GList      *list,
+		       const char *dirname)
 {
 	struct dirent *dent;
 	DIR           *dir;
@@ -1080,24 +1128,25 @@ panel_run_dialog_fill_executables_from (PanelRunDialog *dialog,
 	dir = opendir (dirname);
 
 	if (!dir)
-		return;
+		return list;
 
 	while ((dent = readdir (dir))) {
 		file = g_build_filename (dirname, dent->d_name, NULL);
 
 		if (!access (file, X_OK) &&
 		    !stat (file, &file_stat) && !S_ISDIR (file_stat.st_mode))
-			dialog->executables = g_list_prepend (dialog->executables,
-							      g_strdup (dent->d_name));
+			list = g_list_prepend (list, g_strdup (dent->d_name));
 		
 		g_free (file);
 	}
 
 	closedir (dir);
+	
+	return list;
 }
 
-static void
-panel_run_dialog_fill_executables (PanelRunDialog *dialog)
+static GList *
+fill_executables (GList *list)
 {
 	const char  *path;
 	char       **pathv;
@@ -1106,14 +1155,16 @@ panel_run_dialog_fill_executables (PanelRunDialog *dialog)
 	path = g_getenv ("PATH");
 
 	if (!path || !path [0])
-		return;
+		return list;
 
 	pathv = g_strsplit (path, ":", 0);
 
 	for (i = 0; pathv [i]; i++)
-		panel_run_dialog_fill_executables_from (dialog, pathv[i]);
+		list = fill_executables_from (list, pathv [i]);
 	
 	g_strfreev (pathv);
+	
+	return list;
 }
 
 static void
@@ -1121,9 +1172,44 @@ panel_run_dialog_ensure_completion (PanelRunDialog *dialog)
 {
 	if (!dialog->completion) {
 		dialog->completion = g_completion_new (NULL);
-		panel_run_dialog_fill_executables (dialog);
+		
+		dialog->completion_items = fill_executables (NULL);
+		dialog->completion_items = fill_files_from  (dialog->completion_items, "/", "/", NULL);
+		dialog->completion_items = fill_files_from  (dialog->completion_items, g_get_home_dir(), "", NULL);
+		
+		g_completion_add_items (dialog->completion,
+					dialog->completion_items);
+	}
+}
 
-		g_completion_add_items (dialog->completion, dialog->executables);
+static void
+panel_run_dialog_update_completion (PanelRunDialog *dialog,
+				    const char     *text)
+{
+	g_assert (dialog->completion != NULL);
+	
+	if (text [strlen (text) - 1] == '/') {
+		const char *dirname;
+		char       *freeme;
+		GList      *list;
+		
+		dirname = text;
+		freeme = NULL;
+		
+		if (dirname [0] != '/')
+			dirname = freeme = g_build_filename(g_get_home_dir(), text, NULL);
+		
+		list = fill_files_from (NULL, dirname, text, dialog->completion_items);
+		
+		if (freeme)
+			g_free (freeme);
+		
+		if (list == NULL)
+			return;
+		
+		g_completion_add_items (dialog->completion, list);
+		
+		dialog->completion_items = g_list_concat (dialog->completion_items, list);
 	}
 }
 
@@ -1141,7 +1227,6 @@ entry_event (GtkEditable    *entry,
 	if (event->type == GDK_KEY_PRESS)
 		dialog->use_program_list = FALSE;
 
-
 	/* tab completion */
 	if (event->type == GDK_KEY_PRESS &&
 	    event->keyval == GDK_Tab) {
@@ -1154,6 +1239,9 @@ entry_event (GtkEditable    *entry,
 	    		gtk_editable_select_region (entry, 0, 0);		
 			gtk_editable_set_position (entry, -1);
 
+			panel_run_dialog_update_completion (dialog,
+			    				    gtk_entry_get_text (GTK_ENTRY (entry)));
+			
 			return TRUE;
 		}
 	} else if (event->type == GDK_KEY_PRESS &&
@@ -1189,11 +1277,18 @@ entry_event (GtkEditable    *entry,
 			
 			dialog->completion_started = TRUE;
 
+			if (strcmp(event->string, "/") == 0)
+				panel_run_dialog_update_completion (dialog, nprefix);
+			
 			g_free (nprefix);
 			g_free (prefix);
+			
 			return TRUE;
 		}
 
+		if (strcmp(event->string, "/") == 0)
+			panel_run_dialog_update_completion (dialog, prefix);
+		
 		g_free (prefix);
 	}
 	
@@ -1339,7 +1434,6 @@ pixmap_drag_data_get (GtkWidget          *run_dialog,
 	gboolean          exec = FALSE;
 	const char       *text;
 	char             *uri;
-	char             *path;
 	char             *disk;
 	char             *scheme;
 	
@@ -1384,11 +1478,9 @@ pixmap_drag_data_get (GtkWidget          *run_dialog,
 	
 	g_free (uri);
 
-	path = panel_make_unique_path (g_get_tmp_dir (), ".desktop");
-	gnome_desktop_item_set_location (ditem, path);
+	uri = panel_make_unique_uri (g_get_tmp_dir (), ".desktop");
+	gnome_desktop_item_set_location (ditem, uri);
 
-	uri = gnome_vfs_get_uri_from_local_path (path);
-		
 	if (gnome_desktop_item_save (ditem, NULL, FALSE, NULL))
 		gtk_selection_data_set (selection_data,
 					selection_data->target, 8,
@@ -1396,7 +1488,6 @@ pixmap_drag_data_get (GtkWidget          *run_dialog,
 	gnome_desktop_item_unref (ditem);
 
 	g_free (uri);
-	g_free (path);
 	g_free (disk);
 	g_free (scheme);
 }
@@ -1415,10 +1506,11 @@ panel_run_dialog_setup_pixmap (PanelRunDialog *dialog,
 				  PANEL_STOCK_RUN,
 				  GTK_ICON_SIZE_DIALOG);
 }
-				
+
 static PanelRunDialog *
 panel_run_dialog_new (GdkScreen *screen,
-		      GladeXML  *gui)
+		      GladeXML  *gui,
+		      guint32    activate_time)
 {
 	PanelRunDialog *dialog;
 
@@ -1453,6 +1545,8 @@ panel_run_dialog_new (GdkScreen *screen,
 	set_window_icon_from_stock (GTK_WINDOW (dialog->run_dialog), PANEL_STOCK_RUN);
 
 	gtk_widget_grab_focus (dialog->gtk_entry);
+	gtk_widget_realize (dialog->run_dialog);
+	panel_gdk_x11_window_set_user_time (dialog->run_dialog->window, activate_time);
 	gtk_widget_show (dialog->run_dialog);
 	
 	return dialog;
@@ -1466,29 +1560,27 @@ panel_run_dialog_static_dialog_destroyed (PanelRunDialog *dialog)
 }
 
 void
-panel_run_dialog_present (GdkScreen *screen)
+panel_run_dialog_present (GdkScreen *screen,
+			  guint32    activate_time)
 {
 	GladeXML *gui;
-	char     *glade_file;
 
 	if (panel_lockdown_get_disable_command_line ())
 		return;
 
 	if (static_dialog) {
 		gtk_window_set_screen (GTK_WINDOW (static_dialog->run_dialog), screen);
+		panel_gdk_x11_window_set_user_time (static_dialog->run_dialog->window, activate_time);
 		gtk_window_present (GTK_WINDOW (static_dialog->run_dialog));
 		gtk_widget_grab_focus (static_dialog->gtk_entry);
 		return;
 	}
 
-	if (g_file_test ("panel-run-dialog.glade", G_FILE_TEST_EXISTS))
-		glade_file = "panel-run-dialog.glade";
-	else
-		glade_file = GLADEDIR "/panel-run-dialog.glade";
+	gui = glade_xml_new (GLADEDIR "/panel-run-dialog.glade",
+			     "panel_run_dialog",
+			     NULL);
 
-	gui = glade_xml_new (glade_file, "panel_run_dialog", NULL);
-
-	static_dialog = panel_run_dialog_new (screen, gui);
+	static_dialog = panel_run_dialog_new (screen, gui, activate_time);
 
 	g_signal_connect_swapped (static_dialog->run_dialog, "destroy",
 				  G_CALLBACK (panel_run_dialog_static_dialog_destroyed),
@@ -1496,13 +1588,3 @@ panel_run_dialog_present (GdkScreen *screen)
 
 	g_object_unref (gui);
 }
-
-void
-panel_run_dialog_present_with_text (GdkScreen  *screen,
-				    const char *text)
-{
-	panel_run_dialog_present (screen);
-	
-	if (static_dialog)
-		gtk_entry_set_text (GTK_ENTRY (static_dialog->gtk_entry), text);
-}
