@@ -42,6 +42,7 @@ G_BEGIN_DECLS
 #define PANEL_STOCK_LOGOUT              "panel-logout"
 #define PANEL_STOCK_FORCE_QUIT          "panel-force-quit"
 #define PANEL_STOCK_GNOME_LOGO          "panel-gnome-logo"
+#define PANEL_STOCK_APPLICATIONS        "panel-applications"
 #define PANEL_STOCK_MAIN_MENU           "panel-main-menu"
 #define PANEL_STOCK_DEBIAN              "panel-debian"
 #define PANEL_STOCK_SUSE                "panel-suse"
@@ -61,17 +62,13 @@ G_BEGIN_DECLS
 #define PANEL_STOCK_DRAWER              "panel-drawer"
 #define PANEL_STOCK_APPLETS             "panel-applet"
 #define PANEL_STOCK_DESKTOP             "panel-desktop"
-#define PANEL_STOCK_KDE                 "panel-kde"
-#define PANEL_STOCK_ARROW_RIGHT         "panel-arrow-right"
-#define PANEL_STOCK_ARROW_LEFT          "panel-arrow-left"
-#define PANEL_STOCK_ARROW_UP            "panel-arrow-up"
-#define PANEL_STOCK_ARROW_DOWN          "panel-arrow-down"
 
 /* stock items  - no point in theme the icons one these,
  * they use stock gtk icons and just modify the text
  * for the stock item.
  */
 #define PANEL_STOCK_EXECUTE             "panel-execute"
+#define PANEL_STOCK_CLEAR               "panel-clear"
 
 void        panel_init_stock_icons_and_items (void);
 GtkIconSize panel_menu_icon_get_size         (void);
