@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2003 Sun Microsystems, Inc.
+ * Copyright (C) 2003,2004 Rob Adams
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -172,7 +173,7 @@ panel_struts_allocation_overlapped (PanelStrut   *strut,
 		}
 
 		strut->allocated_strut_start = geometry->y;
-		strut->allocated_strut_end   = geometry->y + geometry->height;
+		strut->allocated_strut_end   = geometry->y + geometry->height - 1;
 	}
 
 	return skip;
@@ -286,20 +287,24 @@ panel_struts_set_window_hint (PanelToplevel *toplevel)
 
 	switch (strut->orientation) {
 	case PANEL_ORIENTATION_TOP:
-		if (topmost && monitor_y > 0)
+		if (monitor_y > 0)
 			strut_size += monitor_y;
+		if (!topmost) strut_size = 0;
 		break;
 	case PANEL_ORIENTATION_BOTTOM:
-		if (bottommost && monitor_y + monitor_height < screen_height)
+		if (monitor_y + monitor_height < screen_height)
 			strut_size += screen_height - (monitor_y + monitor_height);
+		if (!bottommost) strut_size = 0;
 		break;
 	case PANEL_ORIENTATION_LEFT:
 		if (leftmost && monitor_x > 0)
 			strut_size += monitor_x;
+		if (!leftmost) strut_size = 0;
 		break;
 	case PANEL_ORIENTATION_RIGHT:
-		if (rightmost && monitor_x + monitor_width < screen_width)
+		if (monitor_x + monitor_width < screen_width)
 			strut_size += screen_width - (monitor_x + monitor_width);
+		if (!rightmost) strut_size = 0;
 		break;
 	default:
 		g_assert_not_reached ();
@@ -404,26 +409,26 @@ panel_struts_register_strut (PanelToplevel    *toplevel,
 	case PANEL_ORIENTATION_TOP:
 		strut->geometry.x      = strut->strut_start;
 		strut->geometry.y      = monitor_y;
-		strut->geometry.width  = strut->strut_end - strut->strut_start;
+		strut->geometry.width  = strut->strut_end - strut->strut_start + 1;
 		strut->geometry.height = strut->strut_size;
 		break;
 	case PANEL_ORIENTATION_BOTTOM:
 		strut->geometry.x      = strut->strut_start;
 		strut->geometry.y      = monitor_y + monitor_height - strut->strut_size;
-		strut->geometry.width  = strut->strut_end - strut->strut_start;
+		strut->geometry.width  = strut->strut_end - strut->strut_start + 1;
 		strut->geometry.height = strut->strut_size;
 		break;
 	case PANEL_ORIENTATION_LEFT:
 		strut->geometry.x      = monitor_x;
 		strut->geometry.y      = strut->strut_start;
 		strut->geometry.width  = strut->strut_size;
-		strut->geometry.height = strut->strut_end - strut->strut_start;
+		strut->geometry.height = strut->strut_end - strut->strut_start + 1;
 		break;
 	case PANEL_ORIENTATION_RIGHT:
 		strut->geometry.x      = monitor_x + monitor_width - strut->strut_size;
 		strut->geometry.y      = strut->strut_start;
 		strut->geometry.width  = strut->strut_size;
-		strut->geometry.height = strut->strut_end - strut->strut_start;
+		strut->geometry.height = strut->strut_end - strut->strut_start + 1;
 		break;
 	}
 
@@ -466,8 +471,6 @@ panel_struts_update_toplevel_geometry (PanelToplevel *toplevel,
 
 	g_return_val_if_fail (x != NULL, FALSE);
 	g_return_val_if_fail (y != NULL, FALSE);
-	g_return_val_if_fail (width != NULL, FALSE);
-	g_return_val_if_fail (height != NULL, FALSE);
 	
 	if (!(strut = panel_struts_find_strut (toplevel)))
 		return FALSE;
@@ -475,9 +478,9 @@ panel_struts_update_toplevel_geometry (PanelToplevel *toplevel,
 	*x += strut->allocated_geometry.x - strut->geometry.x;
 	*y += strut->allocated_geometry.y - strut->geometry.y;
 
-	if (*width != -1)
+	if (width != NULL && *width != -1)
 		*width  += strut->allocated_geometry.width  - strut->geometry.width;
-	if (*height != -1)
+	if (height != NULL && *height != -1)
 		*height += strut->allocated_geometry.height - strut->geometry.height;
 
 	return TRUE;
